==> yos.core-1.0.0-sources.jar: common/data/SQLProcess.java <==
package common.data;

import beans.department.Department;
import beans.user.User;
import com.alibaba.fastjson.JSONObject;
import common.Controller;
import common.YosException;

import java.util.ArrayList;
import java.util.HashMap;

public class SQLProcess {
    private String sql;
    private String sql_LowerCase;
    private Controller controller;
    private String sqlmodelname_key;

    public static void main(String[] args) throws YosException {
        String sql = "select COUNT(1)                                                      totalCount,\n" +
                "       ifnull(SUM(CASE WHEN t1.status = '待接单' THEN 1 ELSE 0 END), 0) toacceptCount,\n" +
                "       ifnull(SUM(CASE WHEN t1.status = '待开始' THEN 1 ELSE 0 END), 0) tostartCount,\n" +
                "       ifnull(SUM(CASE WHEN t1.status = '进行中' THEN 1 ELSE 0 END), 0) startingCount,\n" +
                "       ifnull(SUM(CASE WHEN t1.status = '已完成' THEN 1 ELSE 0 END), 0) completeCount,\n" +
                "       ifnull(SUM(CASE WHEN t1.status = '暂停' THEN 1 ELSE 0 END), 0)  tostopCount\n" +
                "from sa_workorder t1\n" +
                "         left join sa_serviceorder t2 on t1.sa_serviceorderid = t2.sa_serviceorderid and t1.siteid = t2.siteid\n" +
                "         left join sa_workorder_template t4\n" +
                "                   on t1.sa_workorder_templateid = t4.sa_workorder_templateid and t1.siteid = t4.siteid\n" +
                "         left join sys_enterprise t5 on t5.sys_enterpriseid = t1.sys_enterpriseid and t1.siteid = t5.siteid\n" +
                "         left join sa_agents t6 on t6.sys_enterpriseid = t1.sys_enterpriseid and t1.siteid = t6.siteid\n" +
                "         left join sys_hr t7 on t7.hrid = t2.saler_hrid and t7.siteid = t2.siteid\n" +
                "where t1.siteid = 'TZ'\n" +
                "  and t1.status <> '作废'";
        Controller controller = new Controller(new JSONObject());
        controller.siteid = "TZ";
        controller.userid = 1503;
        new Thread() {
            @Override
            public void run() {
                super.run();
                try {
                    SQLProcess sqlProcess = new SQLProcess(controller, "aa", sql);
                    System.err.println(sqlProcess.getSQL());
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }.start();
        new Thread() {
            @Override
            public void run() {
                super.run();
                try {
                    SQLProcess sqlProcess = new SQLProcess(controller, "aa", sql);
                    System.err.println(sqlProcess.getSQL());
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }.start();

    }

    //  <token,<sqlmodelname_key,tablename>>
    private static HashMap<String, HashMap<String, ArrayList<String>>> map = new HashMap<>();
    private static HashMap<String, RowsMap> useriddatalimitRowsMap = new HashMap<>();

    public SQLProcess(Object o, String sqlmodelname_key, String sql) {
        this.sql = sql.replaceAll("\n", " ").replaceAll(" +", " ");
        this.sql_LowerCase = this.sql.toLowerCase() + " ";
        this.controller = (Controller) o;
        this.sqlmodelname_key = sqlmodelname_key;
    }

    public String getSQL() throws YosException {

        try {
            if (!this.controller.systemclient.equalsIgnoreCase("sysmanage") && this.sql_LowerCase.contains("select")) {
                return getdatalimitSQl();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return this.sql;

    }

    public String getdatalimitSQl() throws YosException {
        synchronized (SQLProcess.class) {
            if (!map.containsKey(this.controller.getAccesstoken())) {
                map.put(this.controller.getAccesstoken(), new HashMap<>());
            }
            if (!map.get(this.controller.getAccesstoken()).containsKey(this.sqlmodelname_key)) {
                map.get(this.controller.getAccesstoken()).put(this.sqlmodelname_key, new ArrayList<>());
            }
            if (map.get(this.controller.getAccesstoken()).get(this.sqlmodelname_key).isEmpty()) {
                Rows rows = this.controller.dbConnect.runSqlQuery("select distinct t2.table_name from sys_userrole t1 inner join sys_role_datalimit t2 on t1.roleid=t2.roleid where t1.siteid='" + this.controller.siteid + "' and t1.userid='" + this.controller.userid + "'");
                for (Row row : rows) {
                    if (this.sql_LowerCase.contains("from " + row.getString("table_name").toLowerCase() + " ")) {
                        map.get(this.controller.getAccesstoken()).get(this.sqlmodelname_key).add(row.getString("table_name").toLowerCase());
                    }
                }
            }
        }
        for (String table_name : map.get(this.controller.getAccesstoken()).get(this.sqlmodelname_key)) {
            RowsMap rowsMap;
            if (useriddatalimitRowsMap.containsKey(this.controller.getAccesstoken())) {
                rowsMap = useriddatalimitRowsMap.get(this.controller.getAccesstoken());
            } else {
                rowsMap = this.controller.dbConnect.runSqlQuery("select distinct t2.sqlstr,t2.table_name from sys_userrole t1 inner join sys_role_datalimit t2 on t1.roleid=t2.roleid where t1.siteid='" + this.controller.siteid + "' and t1.userid='" + this.controller.userid + "'").toRowsMap("table_name");
                useriddatalimitRowsMap.put(this.controller.getAccesstoken(), rowsMap);
            }

            if (rowsMap.containsKey(table_name) && rowsMap.get(table_name).isNotEmpty()) {
                if (this.sql_LowerCase.contains("from " + table_name + " ")) {
                    String where = "1=2";
                    for (Row row : rowsMap.get(table_name)) {
                        String sqlstr = row.getString("sqlstr").trim();

                        if (!sqlstr.equals("")) {
                            if (sqlstr.contains(":hrid")) {
                                sqlstr = sqlstr.replace(":hrid", String.valueOf(this.controller.hrid));
                            }
                            if (sqlstr.contains(":userid")) {
                                sqlstr = sqlstr.replace(":userid", String.valueOf(this.controller.userid));
                            }
                            if (sqlstr.contains(":sys_enterpriseid")) {
                                sqlstr = sqlstr.replace(":sys_enterpriseid", String.valueOf(this.controller.sys_enterpriseid));
                            }
                            if (sqlstr.contains(":departmentid")) {
                                sqlstr = sqlstr.replace(":departmentid", String.valueOf(this.controller.departmentid));
                            }
                            if (sqlstr.contains(":subuserids")) {
                                ArrayList<Long> sublist = User.getSubUserIds(this.controller);
                                sublist.add(this.controller.userid);
                                sqlstr = sqlstr.replace(":subuserids", sublist.toString().replace("[", "(").replace("]", ")"));
                            }
                            if (sqlstr.contains(":subhrids")) {
                                ArrayList<Long> sublist = User.getSubHrIds(this.controller);
                                sublist.add(this.controller.hrid);
                                sqlstr = sqlstr.replace(":subhrids", sublist.toString().replace("[", "(").replace("]", ")"));
                            }
                            if (sqlstr.contains(":subsys_enterpriseids")) {
                                ArrayList<Long> subhridlist = User.getSubHrIds(this.controller);
                                subhridlist.add(controller.hrid);
                                SQLFactory sqlFactory = new SQLFactory("sql:select distinct sys_enterpriseid from sys_enterprise_tradefield where siteid='" + controller.siteid + "' and hrid in $hrids$");
                                sqlFactory.addParameter_in("hrids", subhridlist);
                                ArrayList<Long> sublist = this.controller.dbConnect.runSqlQuery(sqlFactory.getSQL()).toArrayList("sys_enterpriseid", new ArrayList<>());
                                if (sublist.isEmpty()) {
                                    sqlstr = sqlstr.replace(":subsys_enterpriseids", "(null)");
                                } else {
                                    sqlstr = sqlstr.replace(":subsys_enterpriseids", sublist.toString().replace("[", "(").replace("]", ")"));
                                }
                            }
                            if (sqlstr.contains(":subdepartmentids")) {
                                ArrayList<Long> sublist = Department.getSubDepartmentIds(this.controller, this.controller.departmentid);
                                sublist.add(this.controller.departmentid);
                                sqlstr = sqlstr.replace(":subdepartmentids", sublist.toString().replace("[", "(").replace("]", ")"));
                            }
                            where = where + " or (" + sqlstr + ")";
                        }
                    }

                    String s = " select * from " + table_name + " where (" + where + ")";
                    if (this.sql_LowerCase.contains("union")) {
                        if (needAliasTableNameSqlmodelname_keyList.containsKey(this.sqlmodelname_key)) {
                            if (needAliasTableNameSqlmodelname_keyList.get(this.sqlmodelname_key)) {
                                this.sql = this.sql.replaceAll("(?i)from " + table_name + " ", "from (" + s + ") as " + table_name + " ");
                            } else {
                                this.sql = this.sql.replaceAll("(?i)from " + table_name + " ", "from (" + s + ") ");
                            }
                        } else {
                            try {
                                //尝试查询
                                this.controller.dbConnect.runSqlQuery(this.sql.replaceAll("(?i)from " + table_name + " ", "from (" + s + " and 1=2) "));
                                //如果查询成功，则标记该模版不需要进行别名设置
                                needAliasTableNameSqlmodelname_keyList.put(this.sqlmodelname_key, false);
                                this.sql = this.sql.replaceAll("(?i)from " + table_name + " ", "from (" + s + ") ");
                            } catch (Exception e) {
                                if (e.getMessage().contains("Every derived table must have its own alias")) {
                                    //如果抛出没有别名的错误，则加上别名
                                    this.sql = this.sql.replaceAll("(?i)from " + table_name + " ", "from (" + s + ") as " + table_name + " ");
                                    needAliasTableNameSqlmodelname_keyList.put(this.sqlmodelname_key, true);
                                } else {
                                    throw e;
                                }
                            }
                        }
                        // this.sql = this.sql.replaceAll("(?i)from " + table_name + " ", "from (" + s + ") ");
                    } else {
                        if (needAliasTableNameSqlmodelname_keyList.containsKey(this.sqlmodelname_key)) {
                            if (needAliasTableNameSqlmodelname_keyList.get(this.sqlmodelname_key)) {
                                this.sql = this.sql.replaceFirst("(?i)from " + table_name + " ", "from (" + s + ") as " + table_name + " ");
                            } else {
                                this.sql = this.sql.replaceFirst("(?i)from " + table_name + " ", "from (" + s + ") ");
                            }
                        } else {
                            try {
                                //尝试查询
                                this.controller.dbConnect.runSqlQuery(this.sql.replaceFirst("(?i)from " + table_name + " ", "from (" + s + " and 1=2) "));
                                //如果查询成功，则标记该模版不需要进行别名设置
                                needAliasTableNameSqlmodelname_keyList.put(this.sqlmodelname_key, false);
                                this.sql = this.sql.replaceFirst("(?i)from " + table_name + " ", "from (" + s + ") ");
                            } catch (Exception e) {
                                if (e.getMessage().contains("Every derived table must have its own alias")) {
                                    //如果抛出没有别名的错误，则加上别名
                                    this.sql = this.sql.replaceFirst("(?i)from " + table_name + " ", "from (" + s + ") as " + table_name + " ");
                                    needAliasTableNameSqlmodelname_keyList.put(this.sqlmodelname_key, true);
                                } else {
                                    throw e;
                                }
                            }
                        }
                        // this.sql = this.sql.replaceAll("(?i)from " + table_name + " ", "from (" + s + ") ");
                    }
                }
            }
        }
        return this.sql;

    }

    private static HashMap<String, Boolean> needAliasTableNameSqlmodelname_keyList = new HashMap<>();

    public static void cleanPool(String accesstoken) throws YosException {
        map.remove(accesstoken);
        useriddatalimitRowsMap.remove(accesstoken);
    }
}

==> yos.core-2.3.0-sources.jar: common/data/SQLProcess.java <==
package common.data;

import beans.department.Department;
import beans.user.User;
import com.alibaba.fastjson.JSONObject;
import common.Controller;
import common.YosException;

import java.util.ArrayList;
import java.util.HashMap;

public class SQLProcess {
    private String sql;
    private String sql_LowerCase;
    private Controller controller;
    private String sqlmodelname_key;

    public static void main(String[] args) throws YosException {
        String sql = "select COUNT(1)                                                      totalCount,\n" +
                "       ifnull(SUM(CASE WHEN t1.status = '待接单' THEN 1 ELSE 0 END), 0) toacceptCount,\n" +
                "       ifnull(SUM(CASE WHEN t1.status = '待开始' THEN 1 ELSE 0 END), 0) tostartCount,\n" +
                "       ifnull(SUM(CASE WHEN t1.status = '进行中' THEN 1 ELSE 0 END), 0) startingCount,\n" +
                "       ifnull(SUM(CASE WHEN t1.status = '已完成' THEN 1 ELSE 0 END), 0) completeCount,\n" +
                "       ifnull(SUM(CASE WHEN t1.status = '暂停' THEN 1 ELSE 0 END), 0)  tostopCount\n" +
                "from sa_workorder t1\n" +
                "         left join sa_serviceorder t2 on t1.sa_serviceorderid = t2.sa_serviceorderid and t1.siteid = t2.siteid\n" +
                "         left join sa_workorder_template t4\n" +
                "                   on t1.sa_workorder_templateid = t4.sa_workorder_templateid and t1.siteid = t4.siteid\n" +
                "         left join sys_enterprise t5 on t5.sys_enterpriseid = t1.sys_enterpriseid and t1.siteid = t5.siteid\n" +
                "         left join sa_agents t6 on t6.sys_enterpriseid = t1.sys_enterpriseid and t1.siteid = t6.siteid\n" +
                "         left join sys_hr t7 on t7.hrid = t2.saler_hrid and t7.siteid = t2.siteid\n" +
                "where t1.siteid = 'TZ'\n" +
                "  and t1.status <> '作废'";
        Controller controller = new Controller(new JSONObject());
        controller.siteid = "TZ";
        controller.userid = 1503;
        new Thread() {
            @Override
            public void run() {
                super.run();
                try {
                    SQLProcess sqlProcess = new SQLProcess(controller, "aa", sql);
                    System.err.println(sqlProcess.getSQL());
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }.start();
        new Thread() {
            @Override
            public void run() {
                super.run();
                try {
                    SQLProcess sqlProcess = new SQLProcess(controller, "aa", sql);
                    System.err.println(sqlProcess.getSQL());
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }.start();

    }

    //  <token,<sqlmodelname_key,tablename>>
    private static HashMap<String, HashMap<String, ArrayList<String>>> map = new HashMap<>();
    private static HashMap<String, RowsMap> useriddatalimitRowsMap = new HashMap<>();

    public SQLProcess(Object o, String sqlmodelname_key, String sql) {
        this.sql = sql.replaceAll("\n", " ").replaceAll(" +", " ");
        this.sql_LowerCase = this.sql.toLowerCase() + " ";
        this.controller = (Controller) o;
        this.sqlmodelname_key = sqlmodelname_key;
    }

    public String getSQL() throws YosException {
        try {
            if (!this.controller.systemclient.equalsIgnoreCase("sysmanage") && this.sql_LowerCase.contains("select")) {
                return getdatalimitSQl();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return this.sql;

    }

    public String getdatalimitSQl() throws YosException {
        synchronized (SQLProcess.class) {
            if (!map.containsKey(this.controller.getAccesstoken())) {
                map.put(this.controller.getAccesstoken(), new HashMap<>());
            }
            if (!map.get(this.controller.getAccesstoken()).containsKey(this.sqlmodelname_key)) {
                map.get(this.controller.getAccesstoken()).put(this.sqlmodelname_key, new ArrayList<>());
            }
            if (map.get(this.controller.getAccesstoken()).get(this.sqlmodelname_key).isEmpty()) {
                Rows rows = this.controller.dbConnect.runSqlQuery("select distinct t2.table_name from sys_userrole t1 inner join sys_role_datalimit t2 on t1.roleid=t2.roleid where t1.siteid='" + this.controller.siteid + "' and t1.userid='" + this.controller.userid + "'");
                for (Row row : rows) {
                    if (this.sql_LowerCase.contains("from " + row.getString("table_name").toLowerCase() + " ")) {
                        map.get(this.controller.getAccesstoken()).get(this.sqlmodelname_key).add(row.getString("table_name").toLowerCase());
                    }
                }
            }
        }
        for (String table_name : map.get(this.controller.getAccesstoken()).get(this.sqlmodelname_key)) {
            RowsMap rowsMap;
            if (useriddatalimitRowsMap.containsKey(this.controller.getAccesstoken())) {
                rowsMap = useriddatalimitRowsMap.get(this.controller.getAccesstoken());
            } else {
                rowsMap = this.controller.dbConnect.runSqlQuery("select distinct t2.sqlstr,t2.table_name from sys_userrole t1 inner join sys_role_datalimit t2 on t1.roleid=t2.roleid where t1.siteid='" + this.controller.siteid + "' and t1.userid='" + this.controller.userid + "'").toRowsMap("table_name");
                useriddatalimitRowsMap.put(this.controller.getAccesstoken(), rowsMap);
            }

            if (rowsMap.containsKey(table_name) && rowsMap.get(table_name).isNotEmpty()) {
                if (this.sql_LowerCase.contains("from " + table_name + " ")) {
                    String where = "1=2";
                    for (Row row : rowsMap.get(table_name)) {
                        String sqlstr = row.getString("sqlstr").trim();

                        if (!sqlstr.equals("")) {
                            if (sqlstr.contains(":hrid")) {
                                sqlstr = sqlstr.replace(":hrid", String.valueOf(this.controller.hrid));
                            }
                            if (sqlstr.contains(":userid")) {
                                sqlstr = sqlstr.replace(":userid", String.valueOf(this.controller.userid));
                            }
                            if (sqlstr.contains(":sys_enterpriseid")) {
                                sqlstr = sqlstr.replace(":sys_enterpriseid", String.valueOf(this.controller.sys_enterpriseid));
                            }
                            if (sqlstr.contains(":departmentid")) {
                                sqlstr = sqlstr.replace(":departmentid", String.valueOf(this.controller.departmentid));
                            }
                            if (sqlstr.contains(":subuserids")) {
                                ArrayList<Long> sublist = User.getSubUserIds(this.controller);
                                sublist.add(this.controller.userid);
                                sqlstr = sqlstr.replace(":subuserids", sublist.toString().replace("[", "(").replace("]", ")"));
                            }
                            if (sqlstr.contains(":subhrids")) {
                                ArrayList<Long> sublist = User.getSubHrIds(this.controller);
                                sublist.add(this.controller.hrid);
                                sqlstr = sqlstr.replace(":subhrids", sublist.toString().replace("[", "(").replace("]", ")"));
                            }
                            if (sqlstr.contains(":subsys_enterpriseids")) {
                                ArrayList<Long> sublist = User.getSubEnterpriseids(controller);
                                if (sublist.isEmpty()) {
                                    sqlstr = sqlstr.replace(":subsys_enterpriseids", "(null)");
                                } else {
                                    sqlstr = sqlstr.replace(":subsys_enterpriseids", sublist.toString().replace("[", "(").replace("]", ")"));
                                }
                            }
                            if (sqlstr.contains(":subdepartmentids")) {
                                ArrayList<Long> sublist = Department.getSubDepartmentIds(this.controller);
                                sublist.add(this.controller.departmentid);
                                sqlstr = sqlstr.replace(":subdepartmentids", sublist.toString().replace("[", "(").replace("]", ")"));
                            }
                            where = where + " or (" + sqlstr + ")";
                        }
                    }

                    String s = " select * from " + table_name + " where (" + where + ")";
                    if (this.sql_LowerCase.contains("union")) {
                        if (needAliasTableNameSqlmodelname_keyList.containsKey(this.sqlmodelname_key)) {
                            if (needAliasTableNameSqlmodelname_keyList.get(this.sqlmodelname_key)) {
                                this.sql = this.sql.replaceAll("(?i)from " + table_name + " ", "from (" + s + ") as " + table_name + " ");
                            } else {
                                this.sql = this.sql.replaceAll("(?i)from " + table_name + " ", "from (" + s + ") ");
                            }
                        } else {
                            try {
                                //尝试查询
                                this.controller.dbConnect.runSqlQuery(this.sql.replaceAll("(?i)from " + table_name + " ", "from (" + s + " and 1=2) "));
                                //如果查询成功，则标记该模版不需要进行别名设置
                                needAliasTableNameSqlmodelname_keyList.put(this.sqlmodelname_key, false);
                                this.sql = this.sql.replaceAll("(?i)from " + table_name + " ", "from (" + s + ") ");
                            } catch (Exception e) {
                                if (e.getMessage().contains("Every derived table must have its own alias")) {
                                    //如果抛出没有别名的错误，则加上别名
                                    this.sql = this.sql.replaceAll("(?i)from " + table_name + " ", "from (" + s + ") as " + table_name + " ");
                                    needAliasTableNameSqlmodelname_keyList.put(this.sqlmodelname_key, true);
                                } else {
                                    throw e;
                                }
                            }
                        }
                        // this.sql = this.sql.replaceAll("(?i)from " + table_name + " ", "from (" + s + ") ");
                    } else {
                        if (needAliasTableNameSqlmodelname_keyList.containsKey(this.sqlmodelname_key)) {
                            if (needAliasTableNameSqlmodelname_keyList.get(this.sqlmodelname_key)) {
                                this.sql = this.sql.replaceFirst("(?i)from " + table_name + " ", "from (" + s + ") as " + table_name + " ");
                            } else {
                                this.sql = this.sql.replaceFirst("(?i)from " + table_name + " ", "from (" + s + ") ");
                            }
                        } else {
                            try {
                                //尝试查询
                                this.controller.dbConnect.runSqlQuery(this.sql.replaceFirst("(?i)from " + table_name + " ", "from (" + s + " and 1=2) "));
                                //如果查询成功，则标记该模版不需要进行别名设置
                                needAliasTableNameSqlmodelname_keyList.put(this.sqlmodelname_key, false);
                                this.sql = this.sql.replaceFirst("(?i)from " + table_name + " ", "from (" + s + ") ");
                            } catch (Exception e) {
                                if (e.getMessage().contains("Every derived table must have its own alias")) {
                                    //如果抛出没有别名的错误，则加上别名
                                    this.sql = this.sql.replaceFirst("(?i)from " + table_name + " ", "from (" + s + ") as " + table_name + " ");
                                    needAliasTableNameSqlmodelname_keyList.put(this.sqlmodelname_key, true);
                                } else {
                                    throw e;
                                }
                            }
                        }
                        // this.sql = this.sql.replaceAll("(?i)from " + table_name + " ", "from (" + s + ") ");
                    }
                }
            }
        }
        return this.sql;

    }

    private static HashMap<String, Boolean> needAliasTableNameSqlmodelname_keyList = new HashMap<>();

    public static void cleanPool(String accesstoken) throws YosException {
        map.remove(accesstoken);
        useriddatalimitRowsMap.remove(accesstoken);
    }
}
